/*-
 * Copyright 2008 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/errno.h>
#include <sys/soundcard.h>
#include "cx88_audio_capture.h"

cx88_audio_capture::cx88_audio_capture(const char *filename, int &error)
   : cx88_file_capture(filename, error, 0, O_WRONLY)
{
   if (error)
   {
      return;
   }
   int channels = 1;         // 0=mono 1=stereo
   int format = AFMT_S16_LE;
   int rate = 48000;

   /* Tell the sound card that the sound about to be played is stereo. 0=mono 1=stereo */
   if (ioctl(m_file, SNDCTL_DSP_STEREO, &channels) != 0)
   {
      CX88ERR << "Error during SNDCTL_DSP_STEREO: " << strerror(errno) << endl;
      error = errno;
      return;
   }

   if (ioctl(m_file, SNDCTL_DSP_SETFMT, &format) != 0)
   {
      CX88ERR << "Error during SNDCTL_DSP_SETFMT: " << strerror(errno) << endl;
      error = errno;
      return;
   }
   
   /* Set the DSP playback rate, sampling rate of the raw PCM audio */
   if (ioctl(m_file, SNDCTL_DSP_SPEED, &rate) != 0)
   {
      CX88ERR << "Error during SNDCTL_DSP_SPEED: " << strerror(errno) << endl;
      error = errno;
      return;
   }
}
