/*-
 * Copyright 2007 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __CX88_MPEG_DRIVER_H__
#define __CX88_MPEG_DRIVER_H__

#include <pthread.h>
#include <sys/ioctl.h>
#include "dvb_driver.h"
#include "cx88_mpeg_ioctl.h"
#include "cx88_driver.h"

class cx88_mpeg_driver
   : public cx88_driver
{
   public:
      
      cx88_mpeg_driver(
         cx88_xml_config &cx88conf,
         tuner_config &tunerconf,
         const char *devnode,
         capvec &captures,
         int &error);
      
      virtual ~cx88_mpeg_driver(void);
      
   protected:
      
      virtual int capture(const string &channel_string, cx88_capture_params &params);
            
      virtual int get_info(string &info);
      
      virtual double get_signal_strength(void);
      
      virtual int set_channel_options(dvb_channel &channel) 
      {
         return 0;
      }
      
      virtual int set_config_options(cx88_mpeg_config &config) 
      {
         return 0;
      }
      
      virtual int set_capture_options(cx88_mpeg_capture &capture)
      {
         return 0;
      }
      
   private:
      
      int capture(void *buffer, cx88_common_i2c_config &i2config, cx88_mpeg_config &config, cx88_capture_params &params);      
      int change_channel(cx88_common_i2c_config &config, dvb_interface &ifc, cx88_capture_params &params);
      int get_signal(dvb_signal &signal);

};

#endif
