/*-
 * Copyright 2008 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
 
#ifndef __CX88_COMMON_IOCTL_H__
#define __CX88_COMMON_IOCTL_H__

#define CX88_COMMON_MAX_I2C_DEVICES 8
#define CX88_COMMON_MAX_DEVNODE_NAME 32

typedef struct
{
   uint8_t in_i2c_addrs[CX88_COMMON_MAX_I2C_DEVICES]; /* Addresses of I2C devices */
   
   /* Returns names of I2C device files */
   char out_i2c_devnodes[CX88_COMMON_MAX_I2C_DEVICES][CX88_COMMON_MAX_DEVNODE_NAME];
   
   /* Number of occupied slots in the above arrays */
   uint8_t in_num_i2c_devices;
}
cx88_common_i2c_config;

/* GPIO pin control */
typedef struct
{
   uint32_t in_write_mask; /* Select bits in inout_gpio to be written */
   uint32_t in_read_mask; /* Select bits in inout_gpio to be read (any bits selected for writing will be written first) */
   uint32_t inout_gpio; /* GPIO bits (inout_gpio[23:0] -> GPIO pins [23:0] */
} cx88_common_gpio;

#define CX88_COMMON_CARD_ID     _IOR('C', 0, uint32_t)
#define CX88_COMMON_I2C_CONFIG  _IOWR('C', 1, cx88_common_i2c_config)
#define CX88_COMMON_GPIO        _IOWR('C', 2, cx88_common_gpio)

#endif
