#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include "cx88_mpeg_ioctl.h"
#include "tuner_devnode_device.h"
#include "lgh064f.h"
#include "lg3303.h"

void capture(int fd, int buffers, void *bufspace, cx88_mpeg_config &config)
{
   cx88_mpeg_capture capture;
   capture.in_start_byte = CX88_MPEG_DEFAULT_START_BYTE;
   capture.in_start_byte_count = CX88_MPEG_DEFAULT_START_BYTE_COUNT;
   capture.in_options = CX88_MPEG_SERIAL_INPUT | CX88_MPEG_BYTEWIDE_SOP;
   struct in_addr iaddr;
   struct sockaddr_in dest;
   socklen_t sz = 0;
   int outfile = 0;
   size_t msglen = (1500 / config.in_packet_size) * config.in_packet_size;
   if (!inet_aton("127.0.0.1", &iaddr))
   {
      printf("Error creating address\n");
      return;
   }
   if ((outfile = socket(PF_INET, SOCK_DGRAM, 0)) < 0)
   {
      printf("Error opening socket\n");
      return;
   }
   sz = sizeof(dest);
   if (getsockname(outfile, (struct sockaddr *)&dest, &sz) == -1)
   {
      printf("Error getting socket name\n");
      return;
   }
   dest.sin_port = ntohs(8802);
   dest.sin_addr = iaddr;
   if (connect(outfile, (struct sockaddr *)&dest, sz) == -1)
   {
      printf("Error connecting socket\n");
      return;
   }
   /*int outfile = open("out.ts", O_CREAT | O_APPEND | O_TRUNC | O_RDWR, 0777);
   if (outfile < 0)
   {
      printf("Error opening TS output file\n");
      return;  
   }*/
   time_t start_time = time(NULL);
   printf("Beginning capture\n");
   int error = ioctl(fd, CX88_MPEG_CAPTURE, &capture);
   if (error)
   {
      printf("Error during capture ioctl\n");
      return;
   }
   cx88_mpeg_wait wait;
   wait.in_timeout_s = 0;
   do
   {
      error = ioctl(fd, CX88_MPEG_WAIT, &wait);
      if (error)
      {
         printf("Error during wait ioctl\n");
         close(outfile);
         ioctl(fd, CX88_MPEG_STOP);
         return;
      }
      if ((wait.out_status & CX88_MPEG_BUS_ERROR) == CX88_MPEG_BUS_ERROR)
      {
         printf("Fatal error: PCI bus error\n");
      }
      if ((wait.out_status & CX88_MPEG_DMA_PROGRAM) == CX88_MPEG_DMA_PROGRAM)
      {
         printf("Fatal error: DMA program error\n");
      }
      if ((wait.out_status & CX88_MPEG_WAIT_ABORTED) == CX88_MPEG_WAIT_ABORTED)
      {
         printf("Fatal error: Wait aborted\n");
      }
      if ((wait.out_status & CX88_MPEG_CAP_TIMEOUT) == CX88_MPEG_CAP_TIMEOUT)
      {
         printf("Nonfatal error: Capture timed out\n");
      }
      if ((wait.out_status & CX88_MPEG_BAD_PACKET) == CX88_MPEG_BAD_PACKET)
      {
         printf("Nonfatal error: %d bad packets\n", wait.out_num_bad_packets);
      }
      if ((wait.out_status & CX88_MPEG_HW_OVERFLOW) == CX88_MPEG_HW_OVERFLOW)
      {
         printf("Nonfatal error: %d hardware buffer overflows\n", wait.out_num_overflows);
      }
      if (wait.out_status & CX88_MPEG_CAP_FATAL)
      {
         ioctl(fd, CX88_MPEG_STOP);
         close(outfile);
         return;
      }
      if (wait.out_buffer_available == 0 || wait.out_buffer_available > config.out_num_buffers)
      {
         printf("No buffer available\n");
         ioctl(fd, CX88_MPEG_STOP);
         close(outfile);
         return;
      }
      uint32_t bufoffset = (wait.out_buffer_available - 1) * config.bufconfig.out_kernelbuf.out_buffer_size;
      uint32_t bufsize = config.bufconfig.out_kernelbuf.out_mmap_size - bufoffset;
      //write(outfile, (char *)bufspace + bufoffset, (bufsize > (config.out_packets_per_buffer * config.in_packet_size) ? (config.out_packets_per_buffer * config.in_packet_size) : bufsize));
      char *bufptr = (char *)bufspace + bufoffset;
      if (bufsize > (config.out_packets_per_buffer * config.in_packet_size))
      {
         bufsize = config.out_packets_per_buffer * config.in_packet_size;
      }
      uint32_t count = 0, writecount = 0;
      while (count < bufsize)
      {
         writecount = bufsize - count;
         if (writecount > msglen)
         {
            writecount = msglen;
         }
         ssize_t sz2 = write(outfile, bufptr, writecount);
         if (sz2 < 0)
         {
            break;
         }
         bufptr += writecount;
         count += writecount;
      }
   } while ((time(NULL) - start_time) < 100);
   ioctl(fd, CX88_MPEG_STOP);
   close(outfile);
   printf("Packets transferred: %ld\n", wait.out_packet_count);
}

void tune(char *tuner_devnode, char *demod_devnode, int fd, int buffers, void *bufspace, cx88_mpeg_config &config)
{
   int error = 0;
   dvb_channel channel;
   channel.modulation = DVB_MOD_VSB_8;
   channel.frequency_hz = 725000000;
   channel.bandwidth_hz = 6000000;
   tuner_config conf;
   error = conf.load_file("tuner_config");
   if (error)
   {
      printf("Error loading tuner config file\n");
      return;
   }
   tuner_devnode_device tuner_dev(conf, tuner_devnode, error);
   if (error)
   {
      printf("Error creating tuner devnode\n");
      return;
   }
   tuner_devnode_device demod_dev(conf, demod_devnode, error);
   if (error)
   {
      printf("Error creating demod devnode\n");
      return;
   }
   cx88_common_gpio gpio;
   gpio.in_write_mask = 0x1;
   gpio.in_read_mask = 0x0;
   gpio.inout_gpio = 0x0;
   if (ioctl(fd, CX88_COMMON_GPIO, &gpio) != 0)
   {
      printf("Error during GPIO ioctl\n");
      return;
   }
   usleep(100000);
   gpio.in_write_mask = 0x1;
   gpio.in_read_mask = 0x0;
   gpio.inout_gpio = 0x1;
   if (ioctl(fd, CX88_COMMON_GPIO, &gpio) != 0)
   {
      printf("Error during GPIO ioctl 2\n");
      return;  
   }
   usleep(200000);
   lgh064f tuner_driver(conf, tuner_dev);
   lg3303 demod_driver(conf, demod_dev, DVB_IFC_NEG_POL, DVB_INPUT_SERIAL, error);
   if (error)
   {
      printf("Error creating demodulator driver\n");
      return;
   }
   dvb_interface ifc;
   tuner_driver.set_channel(channel, ifc);
   demod_driver.set_channel(channel, ifc);
   printf("Ready to start tuner\n");
   error = tuner_driver.start(10000);
   if (error)
   {
      printf("Error starting tuner\n");
      return;
   }
   printf("Ready to start demod\n");
   error = demod_driver.start(2000);
   dvb_signal signal;
   demod_driver.get_signal(signal);
   printf("Signal stats = %g, %g\n", signal.snr, signal.strength);
   if (error)
   {
      printf("Error starting demod\n");
      tuner_driver.stop();
      return;
   }
   capture(fd, buffers, bufspace, config);
}

int main(void)
{
   cx88_mpeg_config config;
   uint32_t id;
   config.in_packet_size = CX88_MPEG_DEFAULT_PACKET_SIZE;
   config.inout_num_packets = 10000;
   config.in_buftype = CX88_MPEG_BUF_KERNEL;
   int fd = open("/dev/cx88mpeg1", O_RDWR);
   int error = ioctl(fd, CX88_COMMON_CARD_ID, &id);
   if (error)
   {
      printf("Error during card ID ioctl\n");
   }
   printf("Card ID = 0x%x\n", id);
   error = ioctl(fd, CX88_MPEG_CONFIG, &config);
   if (error)
   {
      printf("Error during config ioctl\n");
   }
   cx88_common_i2c_config i2config;
   i2config.in_i2c_addrs[0] = 0x61;
   i2config.in_i2c_addrs[1] = 0x59;
   i2config.in_num_i2c_devices = 2;
   error = ioctl(fd, CX88_COMMON_I2C_CONFIG, &i2config);
   if (error)
   {
      printf("Error during I2C config ioctl\n");
      return 1;  
   }
   printf("Tuner: %s\n", i2config.out_i2c_devnodes[0]);
   printf("Demod: %s\n", i2config.out_i2c_devnodes[1]); 
   printf("Actual num packets: %d\n", config.inout_num_packets);
   printf("Packets per buffer: %d\n", config.out_packets_per_buffer);
   void *buf = mmap(NULL, config.bufconfig.out_kernelbuf.out_mmap_size, PROT_READ, MAP_SHARED, fd, 0);
   if (buf == MAP_FAILED)
   {
      perror("Error mmap'ing device:");
   }
   char tuner_devnode[CX88_COMMON_MAX_DEVNODE_NAME + 5], demod_devnode[CX88_COMMON_MAX_DEVNODE_NAME + 5];
   sprintf(tuner_devnode, "/dev/%s", i2config.out_i2c_devnodes[0]);
   sprintf(demod_devnode, "/dev/%s", i2config.out_i2c_devnodes[1]);
   tune(tuner_devnode, demod_devnode, fd, 400, buf, config);
   if (buf != MAP_FAILED)
   {
      munmap(buf, config.bufconfig.out_kernelbuf.out_mmap_size);
   }
   close(fd);
   return 0;
}

