/*-
 * Copyright 2008 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __CX88_VIDEO_REGS_H__
#define __CX88_VIDEO_REGS_H__

#define VIDEO_REG_Y_PTR1               0x300080
#define VIDEO_REG_Y_PTR2               0x3000C0
#define VIDEO_REG_Y_CNT1               0x300100
#define VIDEO_REG_Y_CNT2               0x300140

#define VIDEO_REG_U_PTR1               0x300084
#define VIDEO_REG_U_PTR2               0x3000C4
#define VIDEO_REG_U_CNT1               0x300104
#define VIDEO_REG_U_CNT2               0x300144

#define VIDEO_REG_V_PTR1               0x300088
#define VIDEO_REG_V_PTR2               0x3000C8
#define VIDEO_REG_V_CNT1               0x300108
#define VIDEO_REG_V_CNT2               0x300148

#define VIDEO_REG_VBI_PTR1             0x30008C
#define VIDEO_REG_VBI_PTR2             0x3000CC
#define VIDEO_REG_VBI_CNT1             0x30010C
#define VIDEO_REG_VBI_CNT2             0x30014C

/* Device status register */
#define VIDEO_REG_STATUS               0x310100
#define VIDEO_STATUS_SCHERR            0x7FFF,16
#define VIDEO_STATUS_NSPLAY            1,7
#define VIDEO_NSPLAY_VCR               0
#define VIDEO_NSPLAY_NORM              1
#define VIDEO_STATUS_VPRES             (1 << 6)
#define VIDEO_STATUS_HLOCK             (1 << 5)
#define VIDEO_STATUS_FIELD             1,4
#define VIDEO_FIELD_ODD                0
#define VIDEO_FIELD_EVEN               1
#define VIDEO_STATUS_NUML              1,3
#define VIDEO_525_LINES                0
#define VIDEO_625_LINES                1
#define VIDEO_STATUS_PLL_LOCK          (1 << 2)
#define VIDEO_STATUS_LUMA_OVERFLOW     (1 << 1)
#define VIDEO_STATUS_CHROMA_OVERFLOW   (1 << 0)

#define VIDEO_REG_INPUT_FORMAT         0x310104
#define VIDEO_IFORM_VIP_INPUT          (1 << 17)
#define VIDEO_IFORM_SVID_CHROMA        (1 << 16)
#define VIDEO_IFORM_INPUT_SOURCE       3, 14
#define VIDEO_IFORM_AGC_ENABLE         (1 << 13)
#define VIDEO_IFORM_NORMAL_AGC         (1 << 12)
#define VIDEO_IFORM_ADAPTIVE_AGC       (1 << 11)
#define VIDEO_IFORM_CHROMA_AGC         (1 << 10)
#define VIDEO_IFORM_COLOR_KILLER       (1 << 9)
#define VIDEO_IFORM_FAST_CHROMA_LOCK   (1 << 8)
#define VIDEO_IFORM_VSYNC_ENABLE       (1 << 7)
#define VIDEO_IFORM_SVIDEO             (1 << 4)
#define VIDEO_IFORM_FORMAT             0xF, 0

#define VIDEO_REG_TDEC                 0x310108
#define VIDEO_TDEC_HL_LP               3, 10
#define VIDEO_TDEC_CCLAMP_DISABLE      (1 << 8)
#define VIDEO_TDEC_DFIELD              (1 << 7)
#define VIDEO_TDEC_EVEN_FIELD          (1 << 6)
#define VIDEO_TDEC_COUNT               0x3F, 0

#define VIDEO_REG_AGC_BURST_DELAY      0x31010C
#define VIDEO_BURST_DELAY              0xFF, 8
#define VIDEO_AGC_DELAY                0xFF, 0      

#define VIDEO_REG_BRIGHT_CONT          0x310110
#define VIDEO_CONTRAST                 0xFF, 8
#define VIDEO_BRIGHTNESS               0xFF, 0

#define VIDEO_REG_UV_SAT               0x310114
#define VIDEO_VSAT                     0xFF, 8
#define VIDEO_USAT                     0xFF, 0

#define VIDEO_REG_HUE                  0x310118

#define VIDEO_REG_HLNOTCH_HTOTAL       0x310120
#define VIDEO_HLOTCH                   3, 11
#define VIDEO_HLNOTCH_4XFSC            0
#define VIDEO_HLNOTCH_SQPIX            1
#define VIDEO_HLNOTCH_135NTSC          2
#define VIDEO_HLNOTCH_135PAL           3
#define VIDEO_HTOTAL                   0x7FF, 0

#define VIDEO_REG_HDELAY_EVEN          0x310124
#define VIDEO_REG_HDELAY_ODD           0x310128

#define VIDEO_REG_VDELAY_ODD           0x31012C
#define VIDEO_REG_VDELAY_EVEN          0x310130

#define VIDEO_REG_VDELAY_VIP_ODD       0x310134
#define VIDEO_REG_VDELAY_VIP_EVEN      0x310138

#define VIDEO_REG_HACTIVE_EVEN         0x31013C
#define VIDEO_REG_HACTIVE_ODD          0x310140

#define VIDEO_REG_VACTIVE_EVEN         0x310144
#define VIDEO_REG_VACTIVE_ODD          0x310148

#define VIDEO_REG_HSCALE_EVEN          0x31014C
#define VIDEO_REG_HSCALE_ODD           0x310150

#define VIDEO_REG_VSCALE_EVEN          0x310154
#define VIDEO_REG_VSCALE_ODD           0x310158

#define VIDEO_REG_FILTER_EVEN          0x31015C
#define VIDEO_REG_FILTER_ODD           0x310160
#define VIDEO_FILTER_P3LPF             (1 << 22)
#define VIDEO_FILTER_P2LPF             (1 << 21)
#define VDIEO_FILTER_P1LPF             (1 << 20)
#define VIDEO_FILTER_CFILT             (1 << 19)
#define VIDEO_FILTER_SNOTCH            (1 << 18)
#define VIDEO_FILTER_CLPF3             (1 << 17)
#define VIDEO_FILTER_CLPF2             (1 << 16)
#define VIDEO_FILTER_CLPF1             3, 14
#define VIDEO_CLPF1_11_TAP             0
#define VIDEO_CLPF1_11_3_TAP           1
#define VIDEO_CLPF1_29_TAP             2
#define VIDEO_FILTER_LNOTCH_DISABLE    (1 << 13)
#define VIDEO_FILTER_LDEC_DISABLE      (1 << 12)
#define VIDEO_FILTER_HFILT             3, 10
#define VIDEO_HFILT_AUTO               0
#define VIDEO_HFILT_CIF                1
#define VIDEO_HFILT_QCIF               2
#define VIDEO_HFILT_ICON               3
#define VIDEO_FILTER_PEAK_ENABLE       (1 << 9)
#define VIDEO_FILTER_PEAK_RESPONSE     3, 7
#define VIDEO_PEAK_2DB                 0
#define VIDEO_PEAK_3_5DB               1
#define VIDEO_PEAK_5DB                 2
#define VIDEO_PEAK_6DB                 3
#define VIDEO_FILTER_COMB              3, 5
#define VIDEO_COMB_FULL                0
#define VIDEO_COMB_CHROMA              1
#define VIDEO_COMB_NONE                3
#define VIDEO_FILTER_FIELDALIGN        (1 << 4)
#define VIDEO_FILTER_VINT              (1 << 3)
#define VIDEO_FILTER_VFILT             7, 0
#define VIDEO_VFILT_INTERP_2TAP        0
#define VIDEO_VFILT_INTERP_3TAP        1
#define VIDEO_VFILT_INTERP_4TAP        2
#define VIDEO_VFILT_INTERP_5TAP        3
#define VIDEO_VFILT_2TAP               4
#define VIDEO_VFILT_3TAP               5
#define VIDEO_VFILT_4TAP               6
#define VIDEO_VFILT_5TAP               7

#define VIDEO_REG_OUTPUT_FORMAT        0x310164
#define VIDEO_OFORM_PREV_REMOD         (1 << 28)
#define VIDEO_OFORM_COMBALT            (1 << 27)
#define VIDEO_OFORM_PAL_INV_PHASE      (1 << 26)
#define VIDEO_OFORM_COMB_RANGE         0x3FF, 16
#define VIDEO_OFORM_DISABLE_IFX        (1 << 15)
#define VIDEO_OFORM_INV_CBFLAG         (1 << 14)
#define VIDEO_OFORM_DISABLE_ADAPT      (1 << 13)
#define VIDEO_OFORM_NARROW_ADAPT       (1 << 12)
#define VIDEO_OFORM_FORCE_CCOMB        (1 << 11)
#define VIDEO_OFORM_FORCE_REMOD        (1 << 10)
#define VIDEO_OFORM_CCOMB_DISABLE      (1 << 9)
#define VIDEO_OFORM_REMOD_DISABLE      (1 << 8)
#define VIDEO_OFORM_YCORE              3, 6
#define VIDEO_YCORE_0                  0
#define VIDEO_YCORE_32                 1
#define VIDEO_YCORE_64                 2
#define VIDEO_YCORE_128                3
#define VIDEO_OFORM_CCORE              3, 4
#define VIDEO_CCORE_0                  0
#define VIDEO_CCORE_7                  1
#define VIDEO_CCORE_15                 2
#define VIDEO_CCORE_31                 3
#define VIDEO_OFORM_FULL_LUMA          (1 << 3)
#define VIDEO_OFORM_HACTEXT            (1 << 2)
#define VIDEO_OFORM_32CLK_HRESET       (1 << 1)

#define VIDEO_REG_PLL                  0x310168
#define VIDEO_PLL_DDS                  (1 << 28)
#define VIDEO_PLL_PRESCALE_DIV         3, 26
#define VIDEO_PRESCALE_DIV_2           0
#define VIDEO_PRESCALE_DIV_5           1
#define VIDEO_PRESCALE_DIV_4           2
#define VIDEO_PRESCALE_DIV_3           3
#define VIDEO_PLL_DIV                  0x3FFFFFF, 0

#define VIDEO_REG_SAMPLE_RATE_CONV     0x310170

#define VIDEO_REG_SCONV_FIFO           0x310174
#define VIDEO_SCFIFO_UPPER             0x3FF, 12
#define VIDEO_SCFIFO_LOWER             0x3FF, 0

#define VIDEO_REG_SUBCARRIER_STEP      0x310178

#define VIDEO_REG_SUBCARRIER_STEP_DR   0x31017C

#define VIDEO_REG_CAPTURE_CONTROL      0x310180
#define VIDEO_CAPTURE_RAW_ALL          (1 << 6)
#define VIDEO_CAPTURE_FILTERED_VBI     (1 << 5)
#define VIDEO_CAPTURE_VBI_ODD          (1 << 4)
#define VIDEO_CAPTURE_VBI_EVEN         (1 << 3)
#define VIDEO_CAPTURE_ODD              (1 << 2)
#define VIDEO_CAPTURE_EVEN             (1 << 1)
#define VIDEO_CAPTURE_FRAME_DITHER     (1 << 0)

#define VIDEO_REG_COLOR_FORMAT         0x310184
#define VIDEO_COLOR_BAR_ENABLE         (1 << 14)
#define VIDEO_COLOR_RGB_DED            (1 << 13)
#define VIDEO_COLOR_GAMMA_DIS          (1 << 12)
#define VIDEO_COLOR_WSWAP_ODD          (1 << 11)
#define VIDEO_COLOR_WSWAP_EVEN         (1 << 10)
#define VIDEO_COLOR_BSWAP_ODD          (1 << 9)
#define VIDEO_COLOR_BSWAP_EVEN         (1 << 8)
#define VIDEO_COLOR_ODD                0xF, 4
#define VIDEO_COLOR_EVEN               0xF, 0

#define VIDEO_REG_VBI_PACKET           0x310188
#define VIDEO_VBI_FRAME_SIZE           0xFFF, 17
#define VIDEO_VBI_VDELAY               0x3F, 11
#define VIDEO_VBI_EXTERN               (1 << 10)
#define VIDEO_VBI_PACKET_SIZE          0x3FF, 0

#define VIDEO_REG_FIELD_COUNTER        0x310190

#define VIDEO_REG_VIP_CONFIG           0x310194
#define VIP_CONFIG_DROP                (1 << 4)
#define VIP_CONFIG_VS_INT              (1 << 3)
#define VIP_CONFIG_CHROMA_SCALE        (1 << 2)
#define VIP_CONFIG_MODE_2              (1 << 1)
#define VIP_CONFIG_INPUT_ENABLE        (1 << 0)

#define VIDEO_REG_VIP_BRIGHT_CONT      0x310198
#define VIP_CONTRAST                   0xFF, 8
#define VIP_BRIGHTNESS                 0xFF, 0

#define VIDEO_REG_VIP_SAT              0x31019C

#define VIDEO_REG_VIP_HSCALE           0x3101A0

#define VIDEO_REG_VIP_VSCALE           0x3101A4
#define VIP_FILT                       7, 13
#define VIP_VFILT_INTERP_2TAP          0
#define VIP_VFILT_INTERP_3TAP          1
#define VIP_VFILT_INTERP_4TAP          2
#define VIP_VFILT_INTERP_5TAP          3
#define VIP_VFILT_2TAP                 4
#define VIP_VFILT_3TAP                 5
#define VIP_VFILT_4TAP                 6
#define VIP_VFILT_5TAP                 7
#define VIP_VSCALE                     0x1FFF, 0

#define VIDEO_REG_VBOS_CONTROL         0x3101A8
#define VBOS_VBIT_NEW                  (1 << 21)
#define VBOS_ACTIVE_LINE_ENABLE        (1 << 20)
#define VBOS_GPCLK_656                 (1 << 19)
#define VBOS_LINE_DELAY                (1 << 18)
#define VBOS_656_ENABLE                (1 << 17)
#define VBOS_10BIT_PIXELS              (1 << 16)
#define VBOS_VBI_ENABLE                (1 << 15)
#define VBOS_VALID_ACTIVE              (1 << 14)
#define VBOS_VBI_BIG_ENDIAN            (1 << 13)
#define VBOS_VBI_FRAME                 (1 << 12)
#define VBOS_HORIZ_ACTIVE              (1 << 11)
#define VBOS_BSTRM_CONTROL_CODES       (1 << 10)
#define VBOS_CONTROL_CODES             (1 << 9)
#define VBOS_16BIT_OUTPUT              (1 << 8)
#define VBOS_VPOLE                     0x7F, 1

#define VIDEO_REG_Y_GP_COUNT           0x31C020
#define VIDEO_REG_Y_GP_COUNT_CONTROL   0x31C030

#define VIDEO_REG_U_GP_COUNT           0x31C024
#define VIDEO_REG_U_GP_COUNT_CONTROL   0x31C034

#define VIDEO_REG_V_GP_COUNT           0x31C028
#define VIDEO_REG_V_GP_COUNT_CONTROL   0x31C038

#define VIDEO_REG_VBI_GP_COUNT         0x31C02C
#define VIDEO_REG_VBI_GP_COUNT_CONTROL 0x31C03C

#define VIDEO_REG_GP_COUNT(bufindex)         (0x31C020 + (bufindex * 4))
#define VIDEO_REG_CP_COUNT_CONTROL(bufindex) (0x31C030 + (bufindex * 4))

#define VIDEO_REG_DMA_CONTROL          0x31C040
#define VIDEO_VBI_RISC_ENABLE          (1 << 7)
#define VIDEO_V_RISC_ENABLE            (1 << 6)
#define VIDEO_U_RISC_ENABLE            (1 << 5)
#define VIDEO_Y_RISC_ENABLE            (1 << 4)
#define VIDEO_VBI_FIFO_ENABLE          (1 << 3)
#define VIDEO_V_FIFO_ENABLE            (1 << 2)
#define VIDEO_U_FIFO_ENABLE            (1 << 1)
#define VIDEO_Y_FIFO_ENABLE            (1 << 0)
#define VIDEO_FIFO_ENABLE(bufindex)    (1 << bufindex)
#define VIDEO_RISC_ENABLE(bufindex)    (1 << (bufindex + 4))

#define VIDEO_REG_XFER_STATUS          0x31C044

#define VIDEO_REG_INT_MASK             0x200050
#define VIDEO_REG_INT_STAT             0x200054
#define VIDEO_REG_INT_MSTAT            0x200058
#define VIDEO_REG_INT_SSTAT            0x20005C
#define VIDEO_INT_PCI_ABORT            (1 << 19)
#define VIDEO_INT_RIP_ERR              (1 << 18)
#define VIDEO_INT_PAR_ERR              (1 << 17)
#define VIDEO_INT_OPCODE_ERR           (1 << 16)
#define VIDEO_INT_VBI_SYNC             (1 << 15)
#define VIDEO_INT_V_SYNC               (1 << 14)
#define VIDEO_INT_U_SYNC               (1 << 13)
#define VIDEO_INT_Y_SYNC               (1 << 12)
#define VIDEO_INT_VBI_OVERFLOW         (1 << 11)
#define VIDEO_INT_V_OVERFLOW           (1 << 10)
#define VIDEO_INT_U_OVERFLOW           (1 << 9)
#define VIDEO_INT_Y_OVERFLOW           (1 << 8)
#define VIDEO_INT_VBI_RISCI2           (1 << 7)
#define VIDEO_INT_V_RISCI2             (1 << 6)
#define VIDEO_INT_U_RISCI2             (1 << 5)
#define VIDEO_INT_Y_RISCI2             (1 << 4)
#define VIDEO_INT_VBI_RISCI1           (1 << 3)
#define VIDEO_INT_V_RISCI1             (1 << 2)
#define VIDEO_INT_U_RISCI1             (1 << 1)
#define VIDEO_INT_Y_RISCI1             (1 << 0)
#define VIDEO_INT_RISCI1(bufindex)     (1 << bufindex)
#define VIDEO_INT_RISCI2(bufindex)     (1 << (bufindex + 4))
#define VIDEO_INT_OVERFLOW(bufindex)   (1 << (bufindex + 8))
#define VIDEO_INT_SYNC(bufindex)       (1 << (bufindex + 12))

#endif
